package com.dfplibrary;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import android.util.Log;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.ReadableNativeArray;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.common.MapBuilder;
import com.facebook.react.uimanager.SimpleViewManager;
import com.facebook.react.uimanager.ThemedReactContext;
import com.facebook.react.uimanager.annotations.ReactProp;
import com.facebook.react.uimanager.events.RCTEventEmitter;
import com.google.android.gms.ads.AdSize;
import com.google.android.gms.ads.admanager.AdManagerAdRequest;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

public class RNDfpBannerViewManager extends SimpleViewManager<RNDfpBannerView> implements RNDfpBannerListener{

    private static final String REACT_CLASS = "RNDFPBanner";

    private static final int COMMAND_RELOAD_BANNER = 1;

    private static final String PROP_TARGETING = "targeting";
    private static final String PROP_AD_SIZES = "adSizes";
    private static final String PROP_DIMENSIONS = "dimensions";
    private static final String PROP_BANNER_SIZE = "bannerSize";
    private static final String PROP_AD_UNIT_ID = "adUnitID";
    private static final String PROP_TEST_DEVICE_ID = "testDeviceID";

    private RCTEventEmitter mEventEmitter;

    public enum Events {
        EVENT_SIZE_CHANGE("onSizeChange"),
        EVENT_RECEIVE_AD("onAdViewDidReceiveAd"),
        EVENT_ERROR("onDidFailToReceiveAdWithError"),
        EVENT_WILL_PRESENT("onAdViewWillPresentScreen"),
        EVENT_WILL_DISMISS("onAdViewWillDismissScreen"),
        EVENT_DID_DISMISS("onAdViewDidDismissScreen"),
        EVENT_ADMOB_EVENT_RECEIVED("onAdmobDispatchAppEvent");

        private final String mName;

        Events(final String name) {
            mName = name;
        }

        @Override
        public String toString() {
            return mName;
        }
    }

    @Override
    public String getName() {
        return REACT_CLASS;
    }


    @Override
    protected RNDfpBannerView createViewInstance(ThemedReactContext themedReactContext) {
        mEventEmitter = themedReactContext.getJSModule(RCTEventEmitter.class);
        return new RNDfpBannerView(themedReactContext, this);
    }

    @Override
    protected void onAfterUpdateTransaction(RNDfpBannerView view) {
        super.onAfterUpdateTransaction(view);
        view.loadBanner();
    }

    @Override
    @Nullable
    public Map<String, Object> getExportedCustomDirectEventTypeConstants() {
        MapBuilder.Builder<String, Object> builder = MapBuilder.builder();
        for (Events event : Events.values()) {
            builder.put(event.toString(), MapBuilder.of("registrationName", event.toString()));
        }
        return builder.build();
    }

    @ReactProp(name = PROP_AD_SIZES)
    public void setAdSizes(final RNDfpBannerView view, final ReadableArray adSizesProp) {
        try {
            if (adSizesProp != null) {
                ArrayList<AdSize> adSizesArrayList = new ArrayList<>();
                ReadableNativeArray nativeadSizesArray = (ReadableNativeArray) adSizesProp;
                for (Object obj : nativeadSizesArray.toArrayList()) {
                    if (obj instanceof String) {
                        AdSize adSize = getAdSizeFromString((String) obj);
                        adSizesArrayList.add(adSize);
                    }

                    if (obj instanceof HashMap) {
                        HashMap dimensions = (HashMap) obj;
                        if (dimensions.containsKey("width") && dimensions.containsKey("height")) {

                            AdSize adSize = new AdSize(
                                    (int) Double.parseDouble(dimensions.get("width").toString()),
                                    (int) Double.parseDouble(dimensions.get("height").toString())
                            );
                            adSizesArrayList.add(adSize);
                        }
                    }
                }
                view.setAdSize(adSizesArrayList.get(0));
            } else {
                view.setAdSize(AdSize.SMART_BANNER);
            }
        } catch (Exception e){
            view.setAdSize(AdSize.SMART_BANNER);
        }
    }

    @ReactProp(name = PROP_TARGETING)
    public void setTargeting(final RNDfpBannerView view, final ReadableMap map){
        view.setTargeting(map);
    }
    
    @ReactProp(name = PROP_DIMENSIONS)
    public void setDimensions(final RNDfpBannerView view, final ReadableMap dimensions) {
        if (dimensions != null && dimensions.hasKey("width") && !dimensions.isNull("width") && dimensions.hasKey("height") && !dimensions.isNull("height")) {
            view.setAdSize(new AdSize(dimensions.getInt("width"), dimensions.getInt("height")));
        } else {
            view.setAdSize(AdSize.SMART_BANNER);
        }
    }

    @ReactProp(name = PROP_BANNER_SIZE)
    public void setBannerSize(final RNDfpBannerView view, final String sizeString) {
        if (sizeString != null && !sizeString.isEmpty()) {
           view.setAdSize(getAdSizeFromString(sizeString));
        } else {
            view.setAdSize(AdSize.SMART_BANNER);
        }
    }

    @ReactProp(name = PROP_AD_UNIT_ID)
    public void setAdUnitID(final RNDfpBannerView view, final String adUnitID) {
        view.setAdUnitId( adUnitID );
    }

    @ReactProp(name = PROP_TEST_DEVICE_ID)
    public void setPropTestDeviceID(final RNDfpBannerView view, final String testDeviceID) {
        WritableArray array = Arguments.createArray();
        array.pushString(testDeviceID);
        view.setTestDevices(array);
    }

    private AdSize getAdSizeFromString(String adSize) {
        switch (adSize) {
            case "banner":
                return AdSize.BANNER;
            case "largeBanner":
                return AdSize.LARGE_BANNER;
            case "mediumRectangle":
                return AdSize.MEDIUM_RECTANGLE;
            case "fullBanner":
                return AdSize.FULL_BANNER;
            case "leaderBoard":
                return AdSize.LEADERBOARD;
            case "smartBannerPortrait":
                return AdSize.SMART_BANNER;
            case "smartBannerLandscape":
                return AdSize.SMART_BANNER;
            case "smartBanner":
                return AdSize.SMART_BANNER;
            default:
                return AdSize.BANNER;
        }
    }

    @Override
    public void onSizeChanged(int id, AdSize size) {
        WritableMap event = Arguments.createMap( );
        event.putDouble( "width", size.getWidth() );
        event.putDouble( "height", size.getHeight() );
        mEventEmitter.receiveEvent( id, Events.EVENT_SIZE_CHANGE.toString( ), event );
    }

    @Override
    public void onGoogleAdLoaded(int id) {
        mEventEmitter.receiveEvent(id, Events.EVENT_RECEIVE_AD.toString(), null);
    }

    @Override
    public void onGoogleAdFailed(int id, int code) {
        WritableMap event = Arguments.createMap();
        switch (code) {
            case AdManagerAdRequest.ERROR_CODE_INTERNAL_ERROR:
                event.putString("error", "ERROR_CODE_INTERNAL_ERROR");
                break;
            case AdManagerAdRequest.ERROR_CODE_INVALID_REQUEST:
                event.putString("error", "ERROR_CODE_INVALID_REQUEST");
                break;
            case AdManagerAdRequest.ERROR_CODE_NETWORK_ERROR:
                event.putString("error", "ERROR_CODE_NETWORK_ERROR");
                break;
            case AdManagerAdRequest.ERROR_CODE_NO_FILL:
                event.putString("error", "ERROR_CODE_NO_FILL");
                break;
        }

        mEventEmitter.receiveEvent(id, Events.EVENT_ERROR.toString(), event);
    }

    @Override
    public void onGoogleAdClosed(int id) {
        mEventEmitter.receiveEvent(id, Events.EVENT_WILL_DISMISS.toString(), null);
    }

    @Override
    public void onGoogleAdOpened(int id) {
        mEventEmitter.receiveEvent(id, Events.EVENT_WILL_PRESENT.toString(), null);
    }

    @Override
    public void onAppEventListener(int id, String name, String info) {
        Log.d( "PublisherAdBanner", String.format( "Received app event (%s, %s)", name, info ));
        WritableMap event = Arguments.createMap( );
        event.putString( name, info );
        mEventEmitter.receiveEvent( id, Events.EVENT_ADMOB_EVENT_RECEIVED.toString( ), event );
    }

    @Override
    public void onError(int id) {
        mEventEmitter.receiveEvent(id, Events.EVENT_ERROR.toString(), null);
    }

    @Nullable
    @Override
    public Map<String, Integer> getCommandsMap() {
        Log.d("React"," View manager getCommandsMap:");
        return MapBuilder.of("reloadBanner", COMMAND_RELOAD_BANNER);
    }

    @Override
    public void receiveCommand(@NonNull RNDfpBannerView view, int commandId, @Nullable ReadableArray args) {
        super.receiveCommand(view, commandId, args);

        switch (commandId) {
            case COMMAND_RELOAD_BANNER: {
                view.loadBanner();
                break;
            }
        }
    }
}
